@@ -2,13 +2,9 @@
 
 import py
 try:
-    from collections.abc import Sequence
+    from collections import Sequence
 except ImportError:
-    try:
-        from collections import Sequence
-    except ImportError:
-        Sequence = list
-
+    Sequence = list
 
 BuiltinAssertionError = py.builtin.builtins.AssertionError
 u = py.builtin._totext
@@ -153,11 +149,10 @@ def assertrepr_compare(config, op, left, right):
             if istext(left) and istext(right):
                 explanation = _notin_text(left, right, verbose)
     except Exception:
-        excinfo = py.code.ExceptionInfo()
         explanation = [
             u('(pytest_assertion plugin: representation of details failed.  '
               'Probably an object has a faulty __repr__.)'),
-            u(excinfo)]
+            u(py.code.ExceptionInfo())]
 
     if not explanation:
         return None
@@ -166,12 +161,18 @@ def assertrepr_compare(config, op, left, right):
 
 
 def _diff_text(left, right, verbose=False):
-    """Return the explanation for the diff between text
+    """Return the explanation for the diff between text or bytes
 
     Unless --verbose is used this will skip leading and trailing
     characters which are identical to keep the diff minimal.
+
+    If the input are bytes they will be safely converted to text.
     """
     explanation = []
+    if isinstance(left, py.builtin.bytes):
+        left = u(repr(left)[1:-1]).replace(r'\n', '\n')
+    if isinstance(right, py.builtin.bytes):
+        right = u(repr(right)[1:-1]).replace(r'\n', '\n')
     if not verbose:
         i = 0  # just in case left or right has zero length
         for i in range(min(len(left), len(right))):
