@@ -1430,6 +1430,78 @@ class TestFixtureMarker:
         reprec = testdir.inline_run()
         reprec.assertoutcome(passed=3)
 
+    def test_scope_session_exc(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+            l = []
+            @pytest.fixture(scope="session")
+            def fix():
+                l.append(1)
+                pytest.skip('skipping')
+
+            def test_1(fix):
+                pass
+            def test_2(fix):
+                pass
+            def test_last():
+                assert l == [1]
+        """)
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(skipped=2, passed=1)
+
+    def test_scope_session_exc_two_fix(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+            l = []
+            m = []
+            @pytest.fixture(scope="session")
+            def a():
+                l.append(1)
+                pytest.skip('skipping')
+            @pytest.fixture(scope="session")
+            def b(a):
+                m.append(1)
+
+            def test_1(b):
+                pass
+            def test_2(b):
+                pass
+            def test_last():
+                assert l == [1]
+                assert m == []
+        """)
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(skipped=2, passed=1)
+
+    def test_scope_exc(self, testdir):
+        testdir.makepyfile(
+            test_foo="""
+                def test_foo(fix):
+                    pass
+            """,
+            test_bar="""
+                def test_bar(fix):
+                    pass
+            """,
+            conftest="""
+                import pytest
+                reqs = []
+                @pytest.fixture(scope="session")
+                def fix(request):
+                    reqs.append(1)
+                    pytest.skip()
+                @pytest.fixture
+                def req_list():
+                    return reqs
+            """,
+            test_real="""
+                def test_last(req_list):
+                    assert req_list == [1]
+            """
+        )
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(skipped=2, passed=1)
+
     def test_scope_module_uses_session(self, testdir):
         testdir.makepyfile("""
             import pytest
@@ -1620,22 +1692,22 @@ class TestFixtureMarker:
         """)
         result = testdir.runpytest("-v")
         result.stdout.fnmatch_lines("""
-            test_mod1.py:1: test_func[s1] PASSED
-            test_mod2.py:1: test_func2[s1] PASSED
-            test_mod2.py:3: test_func3[s1-m1] PASSED
-            test_mod2.py:5: test_func3b[s1-m1] PASSED
-            test_mod2.py:3: test_func3[s1-m2] PASSED
-            test_mod2.py:5: test_func3b[s1-m2] PASSED
-            test_mod1.py:1: test_func[s2] PASSED
-            test_mod2.py:1: test_func2[s2] PASSED
-            test_mod2.py:3: test_func3[s2-m1] PASSED
-            test_mod2.py:5: test_func3b[s2-m1] PASSED
-            test_mod2.py:7: test_func4[m1] PASSED
-            test_mod2.py:3: test_func3[s2-m2] PASSED
-            test_mod2.py:5: test_func3b[s2-m2] PASSED
-            test_mod2.py:7: test_func4[m2] PASSED
-            test_mod1.py:3: test_func1[m1] PASSED
-            test_mod1.py:3: test_func1[m2] PASSED
+            test_mod1.py@1::test_func[s1] PASSED
+            test_mod2.py@1::test_func2[s1] PASSED
+            test_mod2.py@3::test_func3[s1-m1] PASSED
+            test_mod2.py@5::test_func3b[s1-m1] PASSED
+            test_mod2.py@3::test_func3[s1-m2] PASSED
+            test_mod2.py@5::test_func3b[s1-m2] PASSED
+            test_mod1.py@1::test_func[s2] PASSED
+            test_mod2.py@1::test_func2[s2] PASSED
+            test_mod2.py@3::test_func3[s2-m1] PASSED
+            test_mod2.py@5::test_func3b[s2-m1] PASSED
+            test_mod2.py@7::test_func4[m1] PASSED
+            test_mod2.py@3::test_func3[s2-m2] PASSED
+            test_mod2.py@5::test_func3b[s2-m2] PASSED
+            test_mod2.py@7::test_func4[m2] PASSED
+            test_mod1.py@3::test_func1[m1] PASSED
+            test_mod1.py@3::test_func1[m2] PASSED
         """)
 
     def test_class_ordering(self, testdir):
@@ -1672,18 +1744,18 @@ class TestFixtureMarker:
         """)
         result = testdir.runpytest("-vs")
         result.stdout.fnmatch_lines("""
-            test_class_ordering.py:4: TestClass2.test_1[1-a] PASSED
-            test_class_ordering.py:4: TestClass2.test_1[2-a] PASSED
-            test_class_ordering.py:6: TestClass2.test_2[1-a] PASSED
-            test_class_ordering.py:6: TestClass2.test_2[2-a] PASSED
-            test_class_ordering.py:4: TestClass2.test_1[1-b] PASSED
-            test_class_ordering.py:4: TestClass2.test_1[2-b] PASSED
-            test_class_ordering.py:6: TestClass2.test_2[1-b] PASSED
-            test_class_ordering.py:6: TestClass2.test_2[2-b] PASSED
-            test_class_ordering.py:9: TestClass.test_3[1-a] PASSED
-            test_class_ordering.py:9: TestClass.test_3[2-a] PASSED
-            test_class_ordering.py:9: TestClass.test_3[1-b] PASSED
-            test_class_ordering.py:9: TestClass.test_3[2-b] PASSED
+            test_class_ordering.py@4::TestClass2::test_1[1-a] PASSED
+            test_class_ordering.py@4::TestClass2::test_1[2-a] PASSED
+            test_class_ordering.py@6::TestClass2::test_2[1-a] PASSED
+            test_class_ordering.py@6::TestClass2::test_2[2-a] PASSED
+            test_class_ordering.py@4::TestClass2::test_1[1-b] PASSED
+            test_class_ordering.py@4::TestClass2::test_1[2-b] PASSED
+            test_class_ordering.py@6::TestClass2::test_2[1-b] PASSED
+            test_class_ordering.py@6::TestClass2::test_2[2-b] PASSED
+            test_class_ordering.py@9::TestClass::test_3[1-a] PASSED
+            test_class_ordering.py@9::TestClass::test_3[2-a] PASSED
+            test_class_ordering.py@9::TestClass::test_3[1-b] PASSED
+            test_class_ordering.py@9::TestClass::test_3[2-b] PASSED
         """)
 
     def test_parametrize_separated_order_higher_scope_first(self, testdir):
@@ -2087,6 +2159,35 @@ class TestErrors:
             "*1 error*",
         ])
 
+    def test_issue498_fixture_finalizer_failing(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+            @pytest.fixture
+            def fix1(request):
+                def f():
+                    raise KeyError
+                request.addfinalizer(f)
+                return object()
+
+            l = []
+            def test_1(fix1):
+                l.append(fix1)
+            def test_2(fix1):
+                l.append(fix1)
+            def test_3():
+                assert l[0] != l[1]
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines("""
+            *ERROR*teardown*test_1*
+            *KeyError*
+            *ERROR*teardown*test_2*
+            *KeyError*
+            *3 pass*2 error*
+        """)
+
+
+
     def test_setupfunc_missing_funcarg(self, testdir):
         testdir.makepyfile("""
             import pytest
