@@ -124,12 +124,16 @@ class TestMockDecoration:
                 def test_hello(self, abspath):
                     os.path.abspath("hello")
                     abspath.assert_any_call("hello")
+            def mock_basename(path):
+                return "mock_basename"
             @mock.patch("os.path.abspath")
             @mock.patch("os.path.normpath")
+            @mock.patch("os.path.basename", new=mock_basename)
             def test_someting(normpath, abspath, tmpdir):
                 abspath.return_value = "this"
                 os.path.normpath(os.path.abspath("hello"))
                 normpath.assert_any_call("this")
+                assert os.path.basename("123") == "mock_basename"
         """)
         reprec = testdir.inline_run()
         reprec.assertoutcome(passed=2)
@@ -160,6 +164,22 @@ class TestMockDecoration:
         names = [x.nodeid.split("::")[-1] for x in calls]
         assert names == ["test_one", "test_two", "test_three"]
 
+    def test_mock_double_patch_issue473(self, testdir):
+        pytest.importorskip("mock", "1.0.1")
+        testdir.makepyfile("""
+            from mock import patch
+            from pytest import mark
+
+            @patch('os.getcwd')
+            @patch('os.path')
+            @mark.slow
+            class TestSimple:
+                def test_simple_thing(self, mock_path, mock_getcwd):
+                    pass
+        """)
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(passed=1)
+
 
 class TestReRunTests:
     def test_rerun(self, testdir):
@@ -195,3 +215,52 @@ class TestReRunTests:
 def test_pytestconfig_is_session_scoped():
     from _pytest.python import pytestconfig
     assert pytestconfig._pytestfixturefunction.scope == "session"
+
+
+class TestNoselikeTestAttribute:
+    def test_module(self, testdir):
+        testdir.makepyfile("""
+            __test__ = False
+            def test_hello():
+                pass
+        """)
+        reprec = testdir.inline_run()
+        assert not reprec.getfailedcollections()
+        calls = reprec.getreports("pytest_runtest_logreport")
+        assert not calls
+        
+    def test_class_and_method(self, testdir):
+        testdir.makepyfile("""
+            __test__ = True
+            def test_func():
+                pass
+            test_func.__test__ = False
+
+            class TestSome:
+                __test__ = False
+                def test_method(self):
+                    pass
+        """)
+        reprec = testdir.inline_run()
+        assert not reprec.getfailedcollections()
+        calls = reprec.getreports("pytest_runtest_logreport")
+        assert not calls
+
+    def test_unittest_class(self, testdir):
+        testdir.makepyfile("""
+            import unittest
+            class TC(unittest.TestCase):
+                def test_1(self):
+                    pass
+            class TC2(unittest.TestCase):
+                __test__ = False
+                def test_2(self):
+                    pass
+        """)
+        reprec = testdir.inline_run()
+        assert not reprec.getfailedcollections()
+        call = reprec.getcalls("pytest_collection_modifyitems")[0]
+        assert len(call.items) == 1
+        assert call.items[0].cls.__name__ == "TC"
+        
+
