@@ -1,42 +1,44 @@
 import py, pytest
 from _pytest.config import Conftest
 
-def pytest_generate_tests(metafunc):
-    if "basedir" in metafunc.fixturenames:
-        metafunc.addcall(param="global")
-        metafunc.addcall(param="inpackage")
 
-def pytest_funcarg__basedir(request):
-    def basedirmaker(request):
-        d = request.getfuncargvalue("tmpdir")
-        d.ensure("adir/conftest.py").write("a=1 ; Directory = 3")
-        d.ensure("adir/b/conftest.py").write("b=2 ; a = 1.5")
-        if request.param == "inpackage":
-            d.ensure("adir/__init__.py")
-            d.ensure("adir/b/__init__.py")
-        return d
-    return request.cached_setup(
-        lambda: basedirmaker(request), extrakey=request.param)
+@pytest.fixture(scope="module", params=["global", "inpackage"])
+def basedir(request):
+    from _pytest.tmpdir import tmpdir
+    tmpdir = tmpdir(request)
+    tmpdir.ensure("adir/conftest.py").write("a=1 ; Directory = 3")
+    tmpdir.ensure("adir/b/conftest.py").write("b=2 ; a = 1.5")
+    if request.param == "inpackage":
+        tmpdir.ensure("adir/__init__.py")
+        tmpdir.ensure("adir/b/__init__.py")
+    return tmpdir
 
 def ConftestWithSetinitial(path):
     conftest = Conftest()
-    conftest.setinitial([path])
+    conftest_setinitial(conftest, [path])
     return conftest
 
+def conftest_setinitial(conftest, args, confcutdir=None):
+    class Namespace:
+        def __init__(self):
+            self.file_or_dir = args
+            self.confcutdir = str(confcutdir)
+    conftest.setinitial(Namespace())
+
 class TestConftestValueAccessGlobal:
     def test_basic_init(self, basedir):
         conftest = Conftest()
-        conftest.setinitial([basedir.join("adir")])
-        assert conftest.rget("a") == 1
+        p = basedir.join("adir")
+        assert conftest.rget_with_confmod("a", p)[1] == 1
 
     def test_onimport(self, basedir):
         l = []
         conftest = Conftest(onimport=l.append)
-        conftest.setinitial([basedir.join("adir"),
-            '--confcutdir=%s' % basedir])
+        adir = basedir.join("adir")
+        conftest_setinitial(conftest, [adir], confcutdir=basedir)
         assert len(l) == 1
-        assert conftest.rget("a") == 1
-        assert conftest.rget("b", basedir.join("adir", "b")) == 2
+        assert conftest.rget_with_confmod("a", adir)[1] == 1
+        assert conftest.rget_with_confmod("b", adir.join("b"))[1] == 2
         assert len(l) == 2
 
     def test_immediate_initialiation_and_incremental_are_the_same(self, basedir):
@@ -50,37 +52,16 @@ class TestConftestValueAccessGlobal:
         conftest.getconftestmodules(basedir.join('b'))
         assert len(conftest._path2confmods) == snap1 + 2
 
-    def test_default_has_lower_prio(self, basedir):
-        conftest = ConftestWithSetinitial(basedir.join("adir"))
-        assert conftest.rget('Directory') == 3
-        #assert conftest.lget('Directory') == pytest.Directory
-
     def test_value_access_not_existing(self, basedir):
         conftest = ConftestWithSetinitial(basedir)
-        pytest.raises(KeyError, lambda: conftest.rget('a'))
-        #pytest.raises(KeyError, "conftest.lget('a')")
+        with pytest.raises(KeyError):
+            conftest.rget_with_confmod('a', basedir)
 
     def test_value_access_by_path(self, basedir):
         conftest = ConftestWithSetinitial(basedir)
-        assert conftest.rget("a", basedir.join('adir')) == 1
-        #assert conftest.lget("a", basedir.join('adir')) == 1
-        assert conftest.rget("a", basedir.join('adir', 'b')) == 1.5
-        #assert conftest.lget("a", basedir.join('adir', 'b')) == 1
-        #assert conftest.lget("b", basedir.join('adir', 'b')) == 2
-        #assert pytest.raises(KeyError,
-        #    'conftest.lget("b", basedir.join("a"))'
-        #)
-
-    def test_value_access_with_init_one_conftest(self, basedir):
-        conftest = ConftestWithSetinitial(basedir.join('adir'))
-        assert conftest.rget("a") == 1
-        #assert conftest.lget("a") == 1
-
-    def test_value_access_with_init_two_conftests(self, basedir):
-        conftest = ConftestWithSetinitial(basedir.join("adir", "b"))
-        conftest.rget("a") == 1.5
-        #conftest.lget("a") == 1
-        #conftest.lget("b") == 1
+        adir = basedir.join("adir")
+        assert conftest.rget_with_confmod("a", adir)[1] == 1
+        assert conftest.rget_with_confmod("a", adir.join("b"))[1] == 1.5
 
     def test_value_access_with_confmod(self, basedir):
         startdir = basedir.join("adir", "b")
@@ -99,13 +80,13 @@ def test_conftest_in_nonpkg_with_init(tmpdir):
     tmpdir.ensure("adir-1.0/__init__.py")
     ConftestWithSetinitial(tmpdir.join("adir-1.0", "b"))
 
-def test_doubledash_not_considered(testdir):
+def test_doubledash_considered(testdir):
     conf = testdir.mkdir("--option")
     conf.join("conftest.py").ensure()
     conftest = Conftest()
-    conftest.setinitial([conf.basename, conf.basename])
-    l = conftest.getconftestmodules(None)
-    assert len(l) == 0
+    conftest_setinitial(conftest, [conf.basename, conf.basename])
+    l = conftest.getconftestmodules(conf)
+    assert len(l) == 1
 
 def test_issue151_load_all_conftests(testdir):
     names = "code proj src".split()
@@ -114,7 +95,7 @@ def test_issue151_load_all_conftests(testdir):
         p.ensure("conftest.py")
 
     conftest = Conftest()
-    conftest.setinitial(names)
+    conftest_setinitial(conftest, names)
     d = list(conftest._conftestpath2mod.values())
     assert len(d) == len(names)
 
@@ -142,8 +123,8 @@ def test_conftest_global_import(testdir):
 def test_conftestcutdir(testdir):
     conf = testdir.makeconftest("")
     p = testdir.mkdir("x")
-    conftest = Conftest(confcutdir=p)
-    conftest.setinitial([testdir.tmpdir])
+    conftest = Conftest()
+    conftest_setinitial(conftest, [testdir.tmpdir], confcutdir=p)
     l = conftest.getconftestmodules(p)
     assert len(l) == 0
     l = conftest.getconftestmodules(conf.dirpath())
@@ -160,34 +141,18 @@ def test_conftestcutdir(testdir):
 
 def test_conftestcutdir_inplace_considered(testdir):
     conf = testdir.makeconftest("")
-    conftest = Conftest(confcutdir=conf.dirpath())
-    conftest.setinitial([conf.dirpath()])
+    conftest = Conftest()
+    conftest_setinitial(conftest, [conf.dirpath()], confcutdir=conf.dirpath())
     l = conftest.getconftestmodules(conf.dirpath())
     assert len(l) == 1
     assert l[0].__file__.startswith(str(conf))
 
-def test_setinitial_confcut(testdir):
-    conf = testdir.makeconftest("")
-    sub = testdir.mkdir("sub")
-    sub.chdir()
-    for opts in (["--confcutdir=%s" % sub, sub],
-                [sub, "--confcutdir=%s" % sub],
-                ["--confcutdir=.", sub],
-                [sub, "--confcutdir", sub],
-                [str(sub), "--confcutdir", "."],
-    ):
-        conftest = Conftest()
-        conftest.setinitial(opts)
-        assert conftest._confcutdir == sub
-        assert conftest.getconftestmodules(sub) == []
-        assert conftest.getconftestmodules(conf.dirpath()) == []
-
 @pytest.mark.parametrize("name", 'test tests whatever .dotdir'.split())
 def test_setinitial_conftest_subdirs(testdir, name):
     sub = testdir.mkdir(name)
     subconftest = sub.ensure("conftest.py")
     conftest = Conftest()
-    conftest.setinitial([sub.dirpath(), '--confcutdir=%s' % testdir.tmpdir])
+    conftest_setinitial(conftest, [sub.dirpath()], confcutdir=testdir.tmpdir)
     if name not in ('whatever', '.dotdir'):
         assert  subconftest in conftest._conftestpath2mod
         assert len(conftest._conftestpath2mod) == 1
@@ -205,6 +170,26 @@ def test_conftest_confcutdir(testdir):
     result = testdir.runpytest("-h", "--confcutdir=%s" % x, x)
     result.stdout.fnmatch_lines(["*--xyz*"])
 
+def test_conftest_existing_resultlog(testdir):
+    x = testdir.mkdir("tests")
+    x.join("conftest.py").write(py.code.Source("""
+        def pytest_addoption(parser):
+            parser.addoption("--xyz", action="store_true")
+    """))
+    testdir.makefile(ext=".log", result="")  # Writes result.log
+    result = testdir.runpytest("-h", "--resultlog", "result.log")
+    result.stdout.fnmatch_lines(["*--xyz*"])
+
+def test_conftest_existing_junitxml(testdir):
+    x = testdir.mkdir("tests")
+    x.join("conftest.py").write(py.code.Source("""
+        def pytest_addoption(parser):
+            parser.addoption("--xyz", action="store_true")
+    """))
+    testdir.makefile(ext=".xml", junit="")  # Writes junit.xml
+    result = testdir.runpytest("-h", "--junitxml", "junit.xml")
+    result.stdout.fnmatch_lines(["*--xyz*"])
+
 def test_conftest_import_order(testdir, monkeypatch):
     ct1 = testdir.makeconftest("")
     sub = testdir.mkdir("sub")
