@@ -145,6 +145,10 @@ class TestParser:
         assert args.R == True
         assert args.S == False
 
+    def test_parse_removes_line_number_from_positional_arguments(self, parser):
+        args = parser.parse(['path@2::func', 'path2@5::func2[param with @]'])
+        assert getattr(args, parseopt.FILE_OR_DIR) == ['path::func', 'path2::func2[param with @]']
+
     def test_parse_defaultgetter(self):
         def defaultget(option):
             if not hasattr(option, 'type'):
@@ -236,16 +240,6 @@ class TestParser:
         help = parser.optparser.format_help()
         assert '-doit, --func-args  foo' in  help
 
-@pytest.mark.skipif("sys.version_info < (2,5)")
-def test_addoption_parser_epilog(testdir):
-    testdir.makeconftest("""
-        def pytest_addoption(parser):
-            parser.hints.append("hello world")
-            parser.hints.append("from me too")
-    """)
-    result = testdir.runpytest('--help')
-    #assert result.ret != 0
-    result.stdout.fnmatch_lines(["hint: hello world", "hint: from me too"])
 
 @pytest.mark.skipif("sys.version_info < (2,6)")
 def test_argcomplete(testdir, monkeypatch):
